from gi.repository import Gtk
import helper as hl
import os

class BlockSoftCtrlsHandlers:
    def __init__(self, main):
        # Save reference to parent
        self.main = main
        
    def checkBlockAppsToggled(self, togButton):
        # Block all traffic check handler
        state = togButton.get_active() 
        self.main.blockSoft.boxUnblockUsers.set_sensitive(state)
        self.main.blockSoft.scrlWndBlockedApps.set_sensitive(state)
        self.main.blockSoft.boxAddApp.set_sensitive(state)
        self.main.blockSoft.btnRemoveApp.set_sensitive(state)
        
        #self.main.confs.setBlockSoft(state)
    
    def btnSearchAppClicked(self, button):
        chooser = Gtk.FileChooserDialog(title="Block Program", 
                                        action=Gtk.FileChooserAction.OPEN, 
                                        buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
        chooser.set_current_folder(os.path.expanduser("/bin"))
        response = chooser.run()
        if response == Gtk.ResponseType.OK:
            chosenFile = chooser.get_filename()
            self.main.blockSoft.txtApp.set_text(chosenFile)
        chooser.destroy()
        
    def btnAddAppClicked(self, button):
        app = self.main.blockSoft.txtApp.get_text()
        if app != "":
            hl.addRowToScrolledWindow(self.main.blockSoft.scrlWndBlockedApps, app)
            self.main.blockSoft.txtApp.set_text("")
            self.main.confs.addAppToBlocked(app)
    
    def btnRemoveAppClicked(self, button):
        (lst, iterator) = hl.getScrolledWindowSelection(self.main.blockSoft.scrlWndBlockedApps)
        if iterator != None:
            app = lst[iterator][0]
            hl.removeFromScrolledWindowByIter(self.main.blockSoft.scrlWndBlockedApps, iterator)
            self.main.confs.removeAppFromBlocked(app)
            if len(lst) == 0:
                hl.clearScrolledWindow(self.main.blockSoft.scrlWndBlockedUsersApps)
                hl.clearScrolledWindow(self.main.blockSoft.scrlWndAllowedUsersApps)
    
    def btnAddBlockUserAppClicked(self, button):
        (lstApp, iteratorApp) = hl.getScrolledWindowSelection(self.main.blockSoft.scrlWndBlockedApps)
        (lst, iterator) = hl.getScrolledWindowSelection(self.main.blockSoft.scrlWndBlockedUsersApps)
        if iterator != None and iteratorApp != None:
            user = lst[iterator][0]
            app = lstApp[iteratorApp][0]
            hl.removeFromScrolledWindowByIter(self.main.blockSoft.scrlWndBlockedUsersApps, iterator)
            hl.addRowToScrolledWindow(self.main.blockSoft.scrlWndAllowedUsersApps, user)
            self.main.confs.addUserToSoftAllowed(app, user)
            
    
    def btnRemoveUserAppClicked(self, button):
        (lstApp, iteratorApp) = hl.getScrolledWindowSelection(self.main.blockSoft.scrlWndBlockedApps)
        (lst, iterator) = hl.getScrolledWindowSelection(self.main.blockSoft.scrlWndAllowedUsersApps)
        if iterator != None and iteratorApp != None:
            user = lst[iterator][0]
            app = lstApp[iteratorApp][0]
            hl.removeFromScrolledWindowByIter(self.main.blockSoft.scrlWndAllowedUsersApps, iterator)
            hl.addRowToScrolledWindow(self.main.blockSoft.scrlWndBlockedUsersApps, user)
            self.main.confs.removeUserFromSoftAllowed(app, user)
    
    def treeAppsCursorChanged(self, tree):
        if tree.get_selection() != None:
            (lst, iterator) = tree.get_selection().get_selected()
            if lst != None and iterator != None:
                app = lst[iterator][0]
                blockedUsers = self.main.confs.getBlockedUsersFromApp(app)
                diffUsers = hl.diff(hl.getAllNormalUsers(), blockedUsers)
                
                # Clear the scrolled windows
                hl.clearScrolledWindow(self.main.blockSoft.scrlWndBlockedUsersApps)
                hl.clearScrolledWindow(self.main.blockSoft.scrlWndAllowedUsersApps)
                
                # Fill them with the new data
                hl.addRowsToScrolledWindow(self.main.blockSoft.scrlWndBlockedUsersApps, diffUsers)
                hl.addRowsToScrolledWindow(self.main.blockSoft.scrlWndAllowedUsersApps, blockedUsers)
                
class BlockSoftware:
    def __init__(self, main):
        # Save reference to parent
        self.main = main
        
        # Save the gtk builder
        self.builder = main.builder
        
        # GtkCheckButton
        self.checkBlockApps = self.builder.get_object("checkBlockApps")
        
        # GtkEntry
        self.txtApp = self.builder.get_object("txtApp")
        
        # GtkButton
        self.btnSearchApp = self.builder.get_object("btnSearchApp")
        self.btnAddApp = self.builder.get_object("btnAddApp")
        self.btnRemoveApp = self.builder.get_object("btnRemoveApp")
        self.btnAddBlockUserApp = self.builder.get_object("btnAddBlockUserApp")
        self.btnRemoveUserApp = self.builder.get_object("btnRemoveUserApp")
        
        # GtkScrolledWindow
        self.scrlWndBlockedApps = self.builder.get_object("scrlWndBlockedApps")
        self.scrlWndBlockedUsersApps = self.builder.get_object("scrlWndBlockedUsersApps")
        self.scrlWndAllowedUsersApps = self.builder.get_object("scrlWndAllowedUsersApps")
        
        # GtkBox
        self.boxUnblockUsers = self.builder.get_object("boxUnblockUsers")
        self.boxAddApp = self.builder.get_object("boxAddApp")