import helper as hl
from gi.repository import Gtk
from gettext import dgettext as _

class ConfigCtrlsHandlers:
    def __init__(self, main):
        # Save reference to parent
        self.main = main
        
    # Add user button handler
    def btnAddUserClicked(self, button):
        (lst, iterator) = hl.getScrolledWindowSelection(self.main.config.scrlWndAllUsers)
        if iterator != None:
            user = lst[iterator][0]
            hl.removeFromScrolledWindowByIter(self.main.config.scrlWndAllUsers, iterator)
            hl.addRowToScrolledWindow(self.main.config.scrlWndAllowedUsers, user)
            self.main.confs.addUserToAllowed(user)
        
    # Remove user button handler   
    def btnRemoveUserClicked(self, button):
        (lst, iterator) = hl.getScrolledWindowSelection(self.main.config.scrlWndAllowedUsers)
        if iterator != None:
            user = lst[iterator][0]
            hl.removeFromScrolledWindowByIter(self.main.config.scrlWndAllowedUsers, iterator)
            hl.addRowToScrolledWindow(self.main.config.scrlWndAllUsers, user)
            self.main.confs.removeUserFromAllowed(user)
        
    # Enables and disables parental control
    def checkEnableParentalToggled(self, togButton):
        state = togButton.get_active()  
        
        # If active, let the checkBlockTraffic decide
        if(not state):
            self.main.tabBlacklist.set_sensitive(state)
            self.main.tabWhitelist.set_sensitive(state)
            self.main.config.boxTimers.set_sensitive(state)
        else:
            self.main.config.checkAllowTime.toggled()
            self.main.config.checkBlockTraffic.toggled()
            
        self.main.tabBlockSoft.set_sensitive(state)
        self.main.config.boxUserAccess.set_sensitive(state)
        self.main.config.checkAllowTime.set_sensitive(state)
        self.main.config.cboxSensibility.set_sensitive(state)
        self.main.config.checkBlockTraffic.set_sensitive(state)
        
        #self.main.confs.setParental(state)
    
    # Blocks and unblocks all traffic
    def checkBlockTrafficToggled(self, togButton):
        # Block all traffic check handler
        state = togButton.get_active()  
        
        self.main.tabBlacklist.set_sensitive(not state)
        self.main.tabWhitelist.set_sensitive(state)
        
        #self.main.confs.setBlockTraffic(state)
    
    # Enables or enables the timers
    def checkAllowTimeToggled(self, togButton):
        state = togButton.get_active()  
        self.main.config.boxTimers.set_sensitive(state)
    
        #self.main.confs.setTimeEnabled(state)
        
    def cboxSensibilityChanged(self, cBox):
        vals = [50, 100, 160]
        iterator = cBox.get_active()
        
        self.main.confs.cBoxSensibilityIndex = iterator
        self.main.confs.sensibility = vals[iterator]
        
class Configuration:
    def __init__(self, main):
        if(main == None or main.builder == None):
            return
        
        # Save reference to parent
        self.main = main
        
        # Save the gtk builder instance
        self.builder = main.builder
        
        # Get references to the controls
        # GtkCheckBox
        self.checkEnableParental = self.builder.get_object("checkEnableParental")
        self.checkBlockTraffic = self.builder.get_object("checkBlockTraffic")
        self.checkAllowTime = self.builder.get_object("checkAllowTime")
        
        # GtkButton
        self.btnAddUser = self.builder.get_object("btnAddUser")
        self.btnRemoveUser = self.builder.get_object("btnRemoveUser")
        
        # GtkComboBox
        self.cboxSensibility = self.builder.get_object("cboxSensibility")
        
        # GtkScrolledWindow
        self.scrlWndAllUsers = self.builder.get_object("scrlWndAllUsers")
        self.scrlWndAllowedUsers = self.builder.get_object("scrlWndAllowedUsers")
        
        # GtkSpinButton
        self.sbtnStartHours = self.builder.get_object("sbtnStartHours")
        self.sbtnStartMin = self.builder.get_object("sbtnStartMin")
        self.sbtnEndHours = self.builder.get_object("sbtnEndHours")
        self.sbtnEndMin = self.builder.get_object("sbtnEndMin")
        # GtkAdjustment
        self.adjStartHours = self.builder.get_object("adjStartHours")
        self.adjStartMins = self.builder.get_object("adjStartMins")
        self.adjEndHours = self.builder.get_object("adjEndHours")
        self.adjEndMins = self.builder.get_object("adjEndMins")
        
        # GtkBox
        # Box that surrounds the start and end timers
        self.boxTimers = self.builder.get_object("boxTimers")
        # Box that surrounds the user lists 
        self.boxUserAccess = self.builder.get_object("boxUserAccess")

        # Initialize the combobox
        self.initSensibilityCBox()

    def initSensibilityCBox(self):    
        lst = Gtk.ListStore(str)
        lst.append([hl.translate("High")])
        lst.append([hl.translate("Normal")])
        lst.append([hl.translate("Low")])
        cell = Gtk.CellRendererText()
        self.cboxSensibility.pack_start(cell, True)
        self.cboxSensibility.add_attribute(cell, 'text', 0)
        self.cboxSensibility.set_model(lst)
        self.cboxSensibility.set_active(0)