import os
import helper as hl

DANSGUARDIAN_CONFIG_PATH = "/etc/dansguardian"
DANSGUARDIAN_LISTS_PATH = os.path.join(DANSGUARDIAN_CONFIG_PATH, "lists")
DANSGUARDIAN_BANNED_LIST = os.path.join(DANSGUARDIAN_LISTS_PATH, "bannedsitelist")
DANSGUARDIAN_EXCEPTION_LIST = os.path.join(DANSGUARDIAN_LISTS_PATH, "exceptionsitelist")
DANSGUARDIAN_MAIN_FILE = "dansguardian.conf"
DANSGUARDIAN_FILTER_FILE = "dansguardianf1.conf"

BLACKLIST_URLS_FILE = os.path.join(DANSGUARDIAN_LISTS_PATH, "blacklists/urls")
WHITELIST_URLS_FILE = os.path.join(DANSGUARDIAN_LISTS_PATH, "whitelists/urls")

INCLUDE_BLACKLIST = ".Include</etc/dansguardian/lists/blacklists/urls>"
INCLUDE_WHITELIST = ".Include</etc/dansguardian/lists/whitelists/urls>"

class Dansguardian:
    def __init__(self):
        self.mainFile = os.path.join(DANSGUARDIAN_CONFIG_PATH, DANSGUARDIAN_MAIN_FILE)
        self.filterFile = os.path.join(DANSGUARDIAN_CONFIG_PATH, DANSGUARDIAN_FILTER_FILE)
        self.bannedlistFile = DANSGUARDIAN_BANNED_LIST
        self.exceptionlistFile = DANSGUARDIAN_EXCEPTION_LIST
        
        if os.path.exists(DANSGUARDIAN_CONFIG_PATH):
            if hl.fileMatchesRegex(self.mainFile, "^UNCONFIGURED"):
                hl.substituteInFile(self.mainFile, "UNCONFIGURED", "#UNCONFIGURED")
        else:
            raise Exception("Dansguardian configuration folder doesn't exist.")
        
        if not os.path.exists("/etc/dansguardian/lists/blacklists"):
            os.mkdir("/etc/dansguardian/lists/blacklists")
        
        if not os.path.exists("/etc/dansguardian/lists/whitelists"):
            os.mkdir("/etc/dansguardian/lists/whitelists")
            
        if not os.path.isfile(BLACKLIST_URLS_FILE):
            self.clearBlacklist()
            hl.addLineToFile(self.bannedlistFile, "#" + INCLUDE_BLACKLIST + "\n")
             
        if not os.path.isfile(WHITELIST_URLS_FILE):
            self.clearWhitelist()
            hl.addLineToFile(self.exceptionlistFile, "#" + INCLUDE_WHITELIST + "\n")
            
        os.system("dansguardian start")
        
    def dontBlockAllTraffic(self):
        if hl.fileMatchesRegex(self.bannedlistFile, "^\*\*$"):
            hl.substituteInFile(self.bannedlistFile, "\*\*\n", "#**\n")
            
    def blockAllTraffic(self):
        if not hl.fileMatchesRegex(self.bannedlistFile, "^\*\*$"):
            hl.substituteInFile(self.bannedlistFile, "#+\*\*\n", "**\n")
     
    def includeBlacklistUrl(self):
        # Remove the whitelist include
        if hl.fileMatchesRegex(self.exceptionlistFile, "^" + INCLUDE_WHITELIST + "$"):
            hl.substituteInFile(self.exceptionlistFile, INCLUDE_WHITELIST + "\n", "#" + INCLUDE_WHITELIST + "\n")
        # Add the blacklist include
        if not hl.fileMatchesRegex(self.bannedlistFile, "^" + INCLUDE_BLACKLIST + "$"):
            hl.substituteInFile(self.bannedlistFile, "#" + INCLUDE_BLACKLIST + "\n", INCLUDE_BLACKLIST + "\n")
        
    def includeWhitelistUrl(self):
        # Remove the whitelist include
        if hl.fileMatchesRegex(self.bannedlistFile, "^" + INCLUDE_BLACKLIST + "$"):
            hl.substituteInFile(self.bannedlistFile, INCLUDE_BLACKLIST + "\n", "#" + INCLUDE_BLACKLIST + "\n")
        # Add the blacklist include
        if not hl.fileMatchesRegex(self.exceptionlistFile, "^" + INCLUDE_WHITELIST + "$"):
            hl.substituteInFile(self.exceptionlistFile, "#" + INCLUDE_WHITELIST + "\n", INCLUDE_WHITELIST + "\n")
            
    def clearBlacklist(self):
        open(BLACKLIST_URLS_FILE, 'w+').close()       
        
    def clearWhitelist(self):
        open(WHITELIST_URLS_FILE, 'w+').close()
             
    def applyBlacklist(self, siteList):   
        self.includeBlacklistUrl() 
        self.clearBlacklist() 
        f = open(BLACKLIST_URLS_FILE, "a")
        for site in siteList:
            f.write(site + "\n")
        f.close()
                
    def applyWhitelist(self, siteList):
        self.includeWhitelistUrl()
        self.clearWhitelist()
        f = open(WHITELIST_URLS_FILE, "a")
        for site in siteList:
            f.write(site + "\n")
        f.close()
        
    def setSensibilityLevel(self, level):
        hl.substituteInFile(self.filterFile, "naughtynesslimit = \d+", "naughtynesslimit = " + str(level))
        
    def restart(self):
        os.system("dansguardian -Q")
        
    def reload(self):
        os.system("dansguardian -g")