from gi.repository import Gtk
from locale import gettext as _
import subprocess
import pwd
import os
import re
import apt
import stat
from gettext import dgettext as _

locale_domain = "openguard"

def isPackageInstalled(name):
    cache = apt.Cache()
    cache.open()
    return cache[str(name)].is_installed
    
def translate(message):
    return _(locale_domain, message)

#===============================================================================
#    File Related Functions
#===============================================================================
def getApplicationMode(path):
    st = os.stat(path)
    return st.st_mode
    
def blockApplication(path):
    # Remove execute permission
    st = os.stat(path)
    new_mode = st.st_mode
    if new_mode & stat.S_IXGRP:
        new_mode = new_mode ^ stat.S_IXGRP
    if new_mode & stat.S_IXOTH:
        new_mode = new_mode ^ stat.S_IXOTH
    
    os.chmod(path, new_mode)
    return st.st_mode

def restoreApplication(path, oldMode):
    # Restore old permissions
    os.chmod(path, oldMode)
    
    # Remove all acls
    os.system("setfacl -b " + str(path))
    
def unblockUserAppAcl(path, user):
    # Add execute and read permissions to the user
    os.system("setfacl -m u:" + str(user) + ":r-x " + str(path))
    
def unblockUsersAppAcl(path, lst):
    for usr in lst:
        unblockUserAppAcl(path, usr)
    
def removeUserAcl(path, user):
    # Removes the acls from the user
    os.system("setfacl -x u:" + str(user) + " " + str(path))
    
def blockUserProgramAcl(path, user):
    # Remove execute permission
    os.system("setfacl -m u:" + str(user) + ":rw " + str(path))

def blockUsersProgramAcl(path, lst):
    strUsers = ""
    for user in lst:
        strUsers = strUsers + "u:" + str(user) + ":rw" + ","
    os.system("setfacl -m " + strUsers + " " + str(path))

def unblockUserProgramAcl(path, user):
    # Remove acl 
    os.system("setfacl -x u:" + str(user) + " " + str(path))

def unblockUsersProgramAcl(path, lst):
    for user in lst:
        unblockUserProgramAcl(path, user)
    
def unblockProgramAcl(path):
    # Remove all acls
    os.system("setfacl -b " + str(path))

def unblockProgramAclSoft(path):
    for user in getAllNormalUsers():
        unblockUserProgramAcl(path, user)

def areFiles(lstPaths):
    for path in lstPaths:
        if not os.path.isfile(path):
            return False
    return True

# SMALL FILES ONLY
def substituteInFile(subFile, patt, lineTo):
    fh = file(subFile, 'r')
    content = fh.read()
    fh.close()
    replaced = re.sub(patt, lineTo, content, re.MULTILINE)
    fout = file(subFile, 'w+')
    fout.write(replaced)
    fout.close()
    
def matchesRegex(patt, string):
    res = re.search(patt, string, re.MULTILINE)
    return res != None

# SMALL FILES ONLY
def fileMatchesRegex(filePath, patt):
    fh = file(filePath, 'r')
    content = fh.read()
    fh.close()
    return matchesRegex(patt, content)

def addLineToFile(filePath, line):
        with open(filePath, 'a') as fl:
            fl.write(line)

#===============================================================================
#    List Related Functions
#===============================================================================
def diff(a, b):
    b = set(b)
    return [aa for aa in a if aa not in b]

#===============================================================================
#    GTK Controls Related Functions
#===============================================================================
def initScrolledWindow(scrlWnd, cols, *types):
    lst = Gtk.ListStore(*types)
    lst.set_sort_column_id(0, Gtk.SortType.ASCENDING)
    tree = Gtk.TreeView(lst)
    renderer = Gtk.CellRendererText() 
    i = 0
    for col in cols:
        column = Gtk.TreeViewColumn(col, renderer, text=i)
        tree.append_column(column) 
        i = i + 1
    scrlWnd.add(tree)
    tree.show()

def clearScrolledWindow(scrlWnd):
    tree = scrlWnd.get_child()
    model = tree.get_model()
    model.clear()

def removeRowFromScrolledWindow(scrlWnd, row):
    tree = scrlWnd.get_child()
    model = tree.get_model()
    
    for r in model:
        if r[0] == row:
            model.remove(r.iter)

def addRowToScrolledWindow(scrlWnd, entries):
    tree = scrlWnd.get_child()
    model = tree.get_model()
    
    if type(entries) is str:
        model.append([entries])
    else:
        model.append(entries)
    
def addRowsToScrolledWindow(scrlWnd, rows):
    for row in rows:
        addRowToScrolledWindow(scrlWnd, row)
        
def getScrolledWindowSelection(scrlWnd):
    tree = scrlWnd.get_child()
    tpl = tree.get_selection().get_selected()
    return tpl
    
def getScrolledWindowSelectionIter(scrlWnd, rows):
    tree = scrlWnd.get_child()
    tpl = tree.get_selection().get_selected()
    return tpl[1]
    
def removeFromScrolledWindowByIter(scrlWnd, treeIter):
    tree = scrlWnd.get_child()
    model = tree.get_model()
    
    if model.iter_is_valid(treeIter):
        model.remove(treeIter)

#===============================================================================
#    User Related Functions
#===============================================================================
def getMinUid():
    proc = subprocess.Popen("grep \^UID_MIN /etc/login.defs", stdout=subprocess.PIPE, shell=True)
    (out, err) = proc.communicate()
    return out.split()[1]

def getMaxUid():
    proc = subprocess.Popen("grep \^UID_MAX /etc/login.defs", stdout=subprocess.PIPE, shell=True)
    (out, err) = proc.communicate()
    return out.split()[1]

def getAllUsers():
    lstRet = []
    users = pwd.getpwall()
    
    for user in users:
        lstRet.append(user.pw_name)

    return lstRet

def getAllNormalUsers():
    lst = pwd.getpwall()
    lstRet = []
    
    for user in lst:
        if (user.pw_uid >= int(getMinUid()) and user.pw_uid <= int(getMaxUid())):
            lstRet.append(user.pw_name)
            
    return lstRet

def getRootUser():
    return