from gi.repository import Gtk
import configuration as conf
import blacklist as blist
import whitelist as wlist
import blocksoft as soft
import inspect
import helper as hl
from locale import gettext as _
import xml.etree.cElementTree as ET
import shorewall
import dansguardian
import squid
import os

# Main files
PY_FILE_PATH = os.path.dirname(__file__)
PARENTAL_CONFIG_FILE = "/etc/openguard/parental.xml"

DANSGUARDIAN_MAIN_FILE = "/etc/dansguardian/dansguardian.conf";
DANSGUARDIAN_BANNED_SITES_FILE = "/etc/dansguardian/bannedsitelist"

SHOREWALL_DEFAULT_CONFIG_PATH = "/usr/share/doc/shorewall/default-config"
SHOREWALL_CONFIG_PATH = "/etc/shorewall"

SQUID_USER = "proxy"
DANSGUARDIAN_USER = "dansguardian"
ROOT_USER = "root"

GUARDIAN_PORT = 8080
SQUID_PORT = 3128

defaultAllowedUsers = [SQUID_USER, DANSGUARDIAN_USER, ROOT_USER]

class ParentalConfigs:
    def __init__(self, confFile, main):
        self.main = main
        self.enabled = False
        
        # Config members
        self.blockTraffic = False
        self.sensibility = 100
        self.cboxSensibilityIndex = 0
        self.allowedUsers = [SQUID_USER, DANSGUARDIAN_USER, ROOT_USER]
        self.timeEnabled = False
        self.startHour = 0
        self.startMin = 0
        self.endHour = 0
        self.endMin = 0
        
        # whitelist and blacklist
        self.whitelist = []
        self.blacklist = []
        
        # Program block members
        self.softEnabled = False
        self.softlist = []
        self.initialSoftlist = []
        self.blockedApps = []
        
        #self.softNewBlockedUsers = []
        #self.softNewBlockedApps = []
        
        if not os.path.exists(PARENTAL_CONFIG_FILE):
            self.writeSkeleton()
            
        # Parse the config file 
        self.xmlTree = ET.parse(PARENTAL_CONFIG_FILE)
        self.parse(self.xmlTree)
        
    def writeSkeleton(self):
        root = ET.Element('parental', {"enabled" : "false"})
        self.xmlTree = ET.ElementTree(root)
        
        conf = ET.SubElement(root, 'configuration', {"blockTraffic" : "false", "sensibility" : "50"})
        ET.SubElement(conf, 'allow')
        times = ET.SubElement(conf, 'times', {"enabled" : "false"})
        ET.SubElement(times, 'start', {"hour" : "0", "min" : "0"})
        ET.SubElement(times, 'end', {"hour" : "0", "min" : "0"})
       
        ET.SubElement(root, 'blacklist')
        ET.SubElement(root, 'whitelist')
        
        progs = ET.SubElement(root, 'programs', {"enabled" : "false"})
        
        ET.SubElement(progs, 'blocked')
        
        self.xmlTree.write(PARENTAL_CONFIG_FILE, method="xml")
        
    def parse(self, xml):
        root = xml.getroot()
        self.enabled = root.attrib["enabled"] == "true"
        
        conf = root.find("configuration")
        self.blockTraffic = conf.attrib["blockTraffic"] == "true"
        self.sensibility = int(conf.attrib["sensibility"])
        
        for el in conf.find("allow").findall("user"):
            self.allowedUsers.append(el.attrib["name"])

        self.timeEnabled = conf.find("times").attrib["enabled"] == "true"
        start = conf.find("times").find("start")
        self.startHour = int(start.attrib["hour"])
        self.startMin = int(start.attrib["min"])
        
        end = conf.find("times").find("end")
        self.endHour = int(end.attrib["hour"])
        self.endMin = int(end.attrib["min"])
        
        bl = root.find("blacklist")
        for addr in bl.findall("addr"):
            self.blacklist.append(addr.attrib["name"])
            
        wl = root.find("whitelist")
        for addr in wl.findall("addr"):
            self.whitelist.append(addr.attrib["name"])
            
        soft = root.find("programs")
        self.softEnabled = soft.attrib["enabled"] == "true"
        
        for prog in soft.find("blocked").findall("soft"):
            self.blockedApps.append(prog.attrib["path"])
            userList = []
            for user in prog.findall("user"):
                userList.append(user.attrib["name"])
            self.softlist.append((prog.attrib["path"], userList, int(prog.attrib["old_mode"])))
            self.initialSoftlist.append(prog.attrib["path"])
    
    def writeSoftList(self):
        root = self.xmlTree.getroot()
        
        prog = root.find("programs")
        prog.set("enabled", str(self.softEnabled).lower())
        blocked = prog.find("blocked")
        blocked.clear()
        
        for tup in self.softlist:
            soft = ET.Element("soft", {"path" : tup[0], "old_mode" : str(tup[2])})
            blocked.append(soft)
            for user in tup[1]:
                soft.append(ET.Element("user", {"name" : user}))  
        
        self.xmlTree.write(PARENTAL_CONFIG_FILE, method="xml")
     
    def writeSiteList(self):
        root = self.xmlTree.getroot()
        
        bl = root.find("blacklist")
        bl.clear()
        for addr in self.blacklist:
            bl.append(ET.Element("addr", {"name" : addr}))
        
        wl = root.find("whitelist")
        wl.clear()
        for addr in self.whitelist:
            wl.append(ET.Element("addr", {"name" : addr}))
            
        self.xmlTree.write(PARENTAL_CONFIG_FILE, method="xml")
               
    def writeFileScratch(self):
        root = self.xmlTree.getroot()
        root.set("enabled", str(self.enabled).lower())
        
        conf = root.find("configuration")
        conf.set("blockTraffic", str(self.blockTraffic).lower())
        conf.set("sensibility", str(self.sensibility))
        allowList = conf.find("allow")
        allowList.clear()
        for user in hl.diff(self.allowedUsers, defaultAllowedUsers):
            allowList.append(ET.Element("user", {"name" : user}))
        times = conf.find("times")
        times.set("enabled", str(self.timeEnabled).lower())
        times.find("start").set("hour", str(int(self.startHour)))
        times.find("start").set("min", str(int(self.startMin)))
        times.find("end").set("hour", str(int(self.endHour)))
        times.find("end").set("min", str(int(self.endMin)))
        
        bl = root.find("blacklist")
        bl.clear()
        for addr in self.blacklist:
            bl.append(ET.Element("addr", {"name" : addr}))
        
        wl = root.find("whitelist")
        wl.clear()
        for addr in self.whitelist:
            wl.append(ET.Element("addr", {"name" : addr}))
        
        prog = root.find("programs")
        prog.set("enabled", str(self.softEnabled).lower())
        blocked = prog.find("blocked")
        blocked.clear()
        
        for tup in self.softlist:
            soft = ET.Element("soft", {"path" : tup[0], "old_mode" : str(tup[2])})
            blocked.append(soft)
            for user in tup[1]:
                soft.append(ET.Element("user", {"name" : user}))  
        
        self.xmlTree.write(PARENTAL_CONFIG_FILE, method="xml")
        
    def writeEmpty(self):
        # Clear lists
        self.allowedUsers = []
        self.whitelist = []
        self.blacklist = []
        self.softlist = []
        
        # Write file from scratch
        self.writeFileScratch()
  
    def setBlockSoft(self, state):
        self.softEnabled = state;
        
    def setTimeEnabled(self, state):
        self.timeEnabled = state;
        
    def setBlockTraffic(self, state):
        self.blockTraffic = state;
        
    def setParental(self, state):
        self.enabled = state;
    
    def getBlockedUsersFromApp(self, app):
        for tup in self.softlist:
            if app in tup[0]:
                return tup[1]

    def addUserToSoftAllowed(self, app, user):
        for tup in self.softlist:
            if app in tup[0]:
                if not user in tup[1]:
                    tup[1].append(user)
                    if self.enabled and self.softEnabled:
                        hl.unblockUserAppAcl(app, user)
                    self.writeSoftList()
        
    def removeUserFromSoftAllowed(self, app, user):
        for tup in self.softlist:
            if app in tup[0]:
                if user in tup[1]:
                    tup[1].remove(user) 
                    hl.removeUserAcl(app, user)
                    self.writeSoftList()                 
            
    def addAppToBlocked(self, app):
        for tup in self.softlist:
            if app in tup[0]:
                return    
        
        if self.enabled and self.softEnabled:
            oldMode = hl.blockApplication(app)
        else:
            oldMode = hl.getApplicationMode(app)
            
        self.softlist.append((app, [], oldMode))
        self.blockedApps.append(app)
        self.writeSoftList()
        
    def removeAppFromBlocked(self, app):
        for tup in self.softlist:
            if app in tup[0]:
                self.softlist.remove(tup)
                self.blockedApps.remove(app)
                hl.restoreApplication(app, tup[2])
                self.writeSoftList()
                return
        
    def removeUserFromAllowed(self, user):
        if user in self.allowedUsers:
            self.allowedUsers.remove(user)
        
    def addUserToAllowed(self, user):
        if not user in self.allowedUsers:
            self.allowedUsers.append(user)
        
    def addAddressToBlacklist(self, addr):
        if addr != None and addr != "":
            self.blacklist.append(addr)
            self.main.dansguardian.applyBlacklist(self.blacklist)
            self.writeSiteList()
            self.main.dansguardian.reload()
        
    def removeAddressFromBlacklist(self, addr):
        if addr in self.blacklist:
            self.blacklist.remove(addr)
            self.main.dansguardian.applyBlacklist(self.blacklist)
            self.writeSiteList()
            self.main.dansguardian.reload()
            
    def addAddressToWhitelist(self, addr):
        if addr != None and addr != "":
            self.whitelist.append(addr)
            self.main.dansguardian.applyWhitelist(self.whitelist)
            self.writeSiteList()
            self.main.dansguardian.reload()
            
    def removeAddressFromWhitelist(self, addr):
        if addr in self.whitelist:
            self.whitelist.remove(addr)
            self.main.dansguardian.applyWhitelist(self.whitelist)
            self.writeSiteList()
            self.main.dansguardian.reload()
               
    def save(self):
        self.writeFileScratch()

class ParentalCtrlsHandlers:
    def __init__(self, main):
        # Save reference to parent
        self.main = main
        
    def onDeleteWindow(self, *args):
        Gtk.main_quit(*args)
        
    def btnMainOkClicked(self, button):
        self.main.save()

    def btnMainCancelClicked(self, button):
        Gtk.main_quit(0)

    def btnMainHelpClicked(self, button):
        print "btnMainHelpClicked"
        return
    
    def bookMainSwitchPage(self, notebook, page, page_num):
        if page_num != 0:
            self.main.btnMainOk.hide()
        else:
            self.main.btnMainOk.show()
        
class ParentalMain:
    def __init__(self, guiFile):   
        self.builder = Gtk.Builder()
        self.builder.add_from_file(str(guiFile))
        
        # Get object's references
        # GtkWindow
        self.window = self.builder.get_object("wndMain")
        # GtkButton
        self.btnMainOk = self.builder.get_object("btnMainOk")
        self.btnMainCancel = self.builder.get_object("btnMainCancel")
        self.btnMainHelp = self.builder.get_object("btnMainHelp")
        # GtkImage
        self.banner = self.builder.get_object("imgBanner")
        self.banner.set_from_file(os.path.join(PY_FILE_PATH, "ui/banner.png"))
        # GtkBox
        self.tabConfig = self.builder.get_object("tabConfig")
        self.tabBlacklist = self.builder.get_object("tabBlacklist")
        self.tabWhitelist = self.builder.get_object("tabWhitelist")
        self.tabBlockSoft = self.builder.get_object("tabBlockSoft")
        
        # Initialize the program's modules
        self.config = conf.Configuration(self)
        self.blackList = blist.BlackList(self)
        self.whiteList = wlist.WhiteList(self)
        self.blockSoft = soft.BlockSoftware(self)
        
        # Init the handlers 
        self.hndPc = ParentalCtrlsHandlers(self)
        self.hndCf = conf.ConfigCtrlsHandlers(self)
        self.hndBl = blist.BListCtrlsHandlers(self)
        self.hndWl = wlist.WListCtrlsHandlers(self)
        self.hndSt = soft.BlockSoftCtrlsHandlers(self)
        
        # Fill the event handler dictionary
        self.handlerDict = dict()
        self.handlerDict.update(inspect.getmembers(self.hndPc, predicate=inspect.ismethod))
        self.handlerDict.update(inspect.getmembers(self.hndCf, predicate=inspect.ismethod))
        self.handlerDict.update(inspect.getmembers(self.hndBl, predicate=inspect.ismethod))
        self.handlerDict.update(inspect.getmembers(self.hndWl, predicate=inspect.ismethod)) 
        self.handlerDict.update(inspect.getmembers(self.hndSt, predicate=inspect.ismethod))
         
        # Connect all gathered signals
        self.builder.connect_signals(self.handlerDict)
        
        # Load configuration file
        self.confs = ParentalConfigs(PARENTAL_CONFIG_FILE, self)

        # Init scrolled windows
        hl.initScrolledWindow(self.config.scrlWndAllUsers, [hl.translate("Name")], str)
        hl.initScrolledWindow(self.config.scrlWndAllowedUsers,[hl.translate("Name")], str)
        
        hl.initScrolledWindow(self.blackList.scrlWndBlockedAddresses, [hl.translate("Address")], str)        
        hl.initScrolledWindow(self.whiteList.scrlWndAllowedAddresses, [hl.translate("Address")], str)
        
        hl.initScrolledWindow(self.blockSoft.scrlWndBlockedApps, [hl.translate("Application")], str)
        hl.initScrolledWindow(self.blockSoft.scrlWndBlockedUsersApps, [hl.translate("Name")], str)
        hl.initScrolledWindow(self.blockSoft.scrlWndAllowedUsersApps, [hl.translate("Name")], str)
        
        # Updates the state of the controls according configuration
        self.updateControls()
        
        # Initialize shorewall and dansguardian data
        self.shorewall = shorewall.Shorewall()
        self.dansguardian = dansguardian.Dansguardian()
        self.squid = squid.Squid()
        
        # Show the window
        self.window.show_all()
        
        # Hide the help button
        self.btnMainHelp.hide()
        
        # Enter main loop
        Gtk.main()
              
    def save(self):
        # Update timer values
        self.confs.startHour = self.config.adjStartHours.get_value() 
        self.confs.startMin = self.config.adjStartMins.get_value() 
        self.confs.endHour = self.config.adjEndHours.get_value() 
        self.confs.endMin = self.config.adjEndMins.get_value() 

        self.confs.setParental(self.config.checkEnableParental.get_active())        
        self.confs.setBlockTraffic(self.config.checkBlockTraffic.get_active())
        self.confs.setTimeEnabled(self.config.checkAllowTime.get_active())
        self.confs.setBlockSoft(self.blockSoft.checkBlockApps.get_active())
        
        # Save the config file
        self.confs.save()
        
        # Update the dansguardian and shorewall files
        
        # Shorewall
        self.shorewall.clearConfigFile()
        self.shorewall.clearTimeFile()
        if self.confs.enabled:
            self.shorewall.acceptUsers(self.confs.allowedUsers)
            self.shorewall.portRedirect(GUARDIAN_PORT, SQUID_PORT)

            if self.confs.timeEnabled:
                self.shorewall.addTimeControl(self.confs.startHour, self.confs.startMin, self.confs.endHour, self.confs.endMin)

        # Dansguardian
        if self.confs.enabled:
            self.dansguardian.setSensibilityLevel(self.confs.sensibility)
            if self.confs.blockTraffic:
                self.dansguardian.blockAllTraffic()
                #self.dansguardian.applyWhitelist(self.confs.whitelist)
            else:
                self.dansguardian.dontBlockAllTraffic()
                #self.dansguardian.applyBlacklist(self.confs.blacklist)
         
        # Block Programs
        if self.confs.enabled and self.confs.softEnabled:
            for tup in self.confs.softlist:
                hl.blockApplication(tup[0])
                hl.unblockUsersAppAcl(tup[0], tup[1])
                #hl.blockUsersProgramAcl(tup[0], hl.diff(hl.getAllNormalUsers(), tup[1]))
                #hl.unblockUsersProgramAcl(tup[0], tup[1])
        else:
            for tup in self.confs.softlist:
                hl.restoreApplication(tup[0], tup[2])
                #hl.unblockProgramAclSoft(tup[0])
                           
#         for soft in hl.diff(self.confs.initialSoftlist, self.confs.blockedApps):
#             hl.unblockProgramAclSoft(soft)       
#             
#         # Reset the list
#         self.confs.initialSoftlist = []
#         for app in self.confs.blockedApps:
#             self.confs.initialSoftlist.append(app) 
            
        # Restart both dansguardian and shorewall to load configs
        self.shorewall.restart()
        self.dansguardian.reload()
        
        #Gtk.main_quit(0)
    
    def updateControls(self):
        self.config.checkEnableParental.set_active(self.confs.enabled)
        self.config.checkBlockTraffic.set_active(self.confs.blockTraffic)
        self.config.checkAllowTime.set_active(self.confs.timeEnabled)
        self.blockSoft.checkBlockApps.set_active(self.confs.softEnabled)
        
        self.config.cboxSensibility.set_active(self.confs.cboxSensibilityIndex)
              
        # Set timers' value
        self.config.adjStartHours.set_value(self.confs.startHour)
        self.config.adjStartMins.set_value(self.confs.startMin)
        self.config.adjEndHours.set_value(self.confs.endHour)
        self.config.adjEndMins.set_value(self.confs.endMin)
        
        # Add rows to the scrolled windows
        hl.addRowsToScrolledWindow(self.config.scrlWndAllowedUsers, hl.diff(self.confs.allowedUsers, defaultAllowedUsers))
        hl.addRowsToScrolledWindow(self.config.scrlWndAllUsers, hl.diff(hl.getAllNormalUsers(), self.confs.allowedUsers))
        
        hl.addRowsToScrolledWindow(self.blackList.scrlWndBlockedAddresses, self.confs.blacklist)
        hl.addRowsToScrolledWindow(self.whiteList.scrlWndAllowedAddresses, self.confs.whitelist)
        
        hl.addRowsToScrolledWindow(self.blockSoft.scrlWndBlockedApps, self.confs.blockedApps)
        
        # Connect the selection handler
        tree = self.blockSoft.scrlWndBlockedApps.get_child()
        tree.connect("cursor-changed", self.hndSt.treeAppsCursorChanged)

        # Trigger the event to update the dependent controls
        #self.config.checkBlockTraffic.toggled()
        self.blockSoft.checkBlockApps.toggled() 
        self.config.checkAllowTime.toggled()
        self.config.checkBlockTraffic.toggled()
        self.config.checkEnableParental.toggled()
    