import helper as hl
import os
import netifaces

SHOREWALL_DEFAULT_CONFIG_PATH = "/usr/share/doc/shorewall/default-config"
SHOREWALL_DEFAULT_CONFIG_PATH1 = "/usr/share/shorewall/configfiles"
SHOREWALL_CONFIG_PATH = "/etc/shorewall"
SHOREWALL_CONFIG_FILES = ["/etc/shorewall/shorewall.conf", "/etc/shorewall/start", "/etc/shorewall/stop", "/etc/shorewall/init", "/etc/shorewall/rules", "/etc/shorewall/tunnels", "/etc/shorewall/zones"]
SHOREWALL_CONF_FILE = "shorewall.conf"
SHOREWALL_RULES_FILE = "rules"
SHOREWALL_TIME_FILE = "time_control"
SHOREWALL_PARENTAL_CONFIG_FILE = "rules.parental"
SHOREWALL_TIME_CONTROL_FILE = "time_control"

class Shorewall:
    def __init__(self):
        self.rulesFile = os.path.join(SHOREWALL_CONFIG_PATH, SHOREWALL_RULES_FILE)
        self.confFile = os.path.join(SHOREWALL_CONFIG_PATH, SHOREWALL_CONF_FILE)
        self.configFile = os.path.join(SHOREWALL_CONFIG_PATH, SHOREWALL_PARENTAL_CONFIG_FILE)
        self.timeFile = os.path.join(SHOREWALL_CONFIG_PATH, SHOREWALL_TIME_FILE)
        self.startFile = os.path.join(SHOREWALL_CONFIG_PATH, "start")
        self.zonesFile = os.path.join(SHOREWALL_CONFIG_PATH, "zones")
        self.interfacesFile = os.path.join(SHOREWALL_CONFIG_PATH, "interfaces")
        self.policyFile = os.path.join(SHOREWALL_CONFIG_PATH, "policy")
        if not self.verifyConfigFolder():
            if os.path.exists(SHOREWALL_DEFAULT_CONFIG_PATH):
                os.system("cp " + os.path.join(SHOREWALL_DEFAULT_CONFIG_PATH, "*") + " " + SHOREWALL_CONFIG_PATH)
            elif os.path.exists(SHOREWALL_DEFAULT_CONFIG_PATH1):
                os.system("cp " + os.path.join(SHOREWALL_DEFAULT_CONFIG_PATH1, "*") + " " + SHOREWALL_CONFIG_PATH)
            else:
                raise Exception("Shorewall's configuration folder doesn't exist or doesn't have the required configuration files.")
        self.initialConfig()
        self.includeConfigFile()
        self.includeTimeControlFile()
    
    def initialConfig(self):
        if not hl.fileMatchesRegex(self.zonesFile, "^net\tipv4"):
            hl.addLineToFile(self.zonesFile, "net\tipv4\n")
            
        open(self.interfacesFile, "w+")
        for interface in netifaces.interfaces():
            if interface != "lo" and interface != "lo0":
                hl.addLineToFile(self.interfacesFile, "net\t" + interface + "\n")
            
        if not hl.fileMatchesRegex(self.policyFile, "^fw\tnet\tACCEPT"):
            hl.addLineToFile(self.policyFile, "fw\tnet\tACCEPT\nnet\tall\tDROP\tinfo\nall\tall\tREJECT\tinfo\n")
            
        if hl.fileMatchesRegex(self.startFile, "^return 0"):
            hl.substituteInFile(self.startFile, "return 0", "")
    
    def includeTimeControlFile(self):
        if not hl.fileMatchesRegex(self.startFile, "^INCLUDE " + SHOREWALL_TIME_CONTROL_FILE + "\n"):
            hl.addLineToFile(self.startFile, "INCLUDE " + SHOREWALL_TIME_CONTROL_FILE + "\n")
        
    def includeConfigFile(self):
        if not hl.fileMatchesRegex(self.rulesFile, "^INCLUDE\t" + SHOREWALL_PARENTAL_CONFIG_FILE + "\n"):
            hl.addLineToFile(self.rulesFile, "INCLUDE\t" + SHOREWALL_PARENTAL_CONFIG_FILE + "\n")
    
    def clearTimeFile(self):
        open(self.timeFile, 'w+')
 
    def addTimeControl(self, startHour, startMin, endHour, endMin):
        shour = "0" + str(int(startHour)) if startHour < 10 else str(int(startHour))
        smin = "0" + str(int(startMin)) if startMin < 10 else str(int(startMin))
        ehour = "0" + str(int(endHour)) if endHour < 10 else str(int(endHour))
        emin = "0" + str(int(endMin)) if endMin < 10 else str(int(endMin))
        fh = open(self.timeFile, 'a')
        fh.write("run_iptables -I net2fw -j DROP -m time --timestart 00:00 --timestop " + shour + ":" + smin + " --kerneltz" + "\n")
        fh.write("run_iptables -I net2fw -j DROP -m time --timestart " + ehour + ":" + emin + " --timestop 23:59 --kerneltz" + "\n")
        fh.write("run_iptables -I fw2net -j DROP -m time --timestart 00:00 --timestop " + shour + ":" + smin + " --kerneltz" + "\n")
        fh.write("run_iptables -I fw2net -j DROP -m time --timestart " + ehour + ":" + emin + " --timestop 23:59 --kerneltz" + "\n")
        fh.close()
        
    def clearConfigFile(self):
        open(self.configFile, 'w+')
        
    def portRedirect(self, httpPort, proxyPort):
        fh = open(self.configFile, 'a')
        fh.write("REDIRECT\tfw\t" + str(httpPort) + "\ttcp\t" + str(proxyPort) + "\t-" + "\n")
        fh.write("REDIRECT\tfw\t" + str(httpPort) + "\ttcp\thttp\t-" + "\n")
        fh.close()
    
    def acceptUsers(self, allowedUsers):
        fh = open(self.configFile, 'a')
        for user in allowedUsers:
            if user == "dansguardian":
                fh.write("ACCEPT+ fw\tnet\ttcp\t3128\t-\t-\t-\t" + str(user) + "\n")
            else:
                fh.write("ACCEPT+ fw\tnet\ttcp\thttp\t-\t-\t-\t" + str(user) + "\n")
        fh.close()
    
    def start(self):
        os.system("shorewall start")
        
    def restart(self):
        os.system("shorewall restart")
        
    def stop(self):
        os.system("shorewall stop")
        
    def verifyConfigFolder(self):
        return hl.areFiles(SHOREWALL_CONFIG_FILES)